# DESeq2

## Libraries

library("DESeq2")
library("tximport")
library("rhdf5")
library("ggplot2")

# ## Tests 

# inKallistoResults <- "../../runs/kallisto/kallisto_results"
# inMetadataFile <- "../../runs/isaSampleToRawDataSeq/rnaseq-samples.csv"
# inMetadataSample <- "Input [Source Name]"
# inMetadataFactorList <- list("Factor [Photosynthesis mode]")

### Read arguments from CLI

args <- commandArgs(trailingOnly = T)

inKallistoResults <- args[1]
inMetadataFile <- args[2]
inMetadataSample <- args[3]
inMetadataFactorList <- args[4]

## Import kallisto count data

files <- dir(inKallistoResults, recursive = T, full.names = T ,"abundance.h5")
names(files) <- dir(inKallistoResults)

txi <- tximport(files, type = "kallisto", txOut = TRUE)

head(txi$counts)

## Read sample metadata

samples_metadata <- as.data.frame(read.csv(inMetadataFile, check.names=FALSE))
samples <- samples_metadata[order(samples_metadata[[inMetadataSample]]), c(inMetadataSample, unlist(inMetadataFactorList))]

rownames(samples) <- samples[,inMetadataSample]

factors <- sapply(inMetadataFactorList, function(x) x[[1]])

## Annoying workaround to prevent formula error with special chars in column headers
colnames(samples) <- make.names(colnames(samples))
factors <- make.names(factors)

design_formula <- as.formula(paste("~", paste(rev(factors), collapse = " + ")))

## DESeq

dds <- DESeqDataSetFromTximport(txi, colData = samples, design = design_formula)

dds <- DESeq(dds)

## Outputs

### Extract results

res <- results(dds)
write.csv(res, file = "results_stats.csv", quote = TRUE)

### Generate and save default plots

#### MA Plot

svg("results_ma-plot.svg")
  p1 <- plotMA(res, ylim=c(-2,2))
dev.off()

#### PCA

vsd <- vst(dds, blind=FALSE)
pcaData <- plotPCA(vsd, intgroup=factors, returnData=TRUE)
percentVar <- round(100 * attr(pcaData, "percentVar"))

p2 <- ggplot(pcaData, aes(PC1, PC2, color=.data[[factors[[1]]]])) +
  geom_point(size=3) +
  xlab(paste0("PC1: ",percentVar[1],"% variance")) +
  ylab(paste0("PC2: ",percentVar[2],"% variance")) +
  theme(aspect.ratio = 1) +
  theme(panel.background = element_blank()) +  #background of the plotting area
  theme(panel.grid.major = element_blank(), panel.grid.minor = element_blank()) +
  coord_fixed()

svg("results_pca-plot.svg", width = 8, height = 8)
  print(p2)
dev.off()
